# -*- coding: utf-8 -*-
"""
Created on Sun Jan 17 14:02:00 2021

@author: Ali Jazayeri
"""


class Edge():
    def __init__(self, frm, to, lbl):

        self.frm = frm
        self.to = to
        self.lbl = lbl