# -*- coding: utf-8 -*-
"""
Created on Sun Jan 17 13:54:00 2021

@author: Ali Jazayeri
"""


class nil(object):
    def __init__(self):
        self.color = "black"
        self.key = None
        self.maximum = None
        self.parent = None